--Author: Alexey Tsepelev
--Web: sumnix.net
--History:
	-- 0.1 It works with spline's knots
	-- 0.1.1 Now it works with tangents
	-- 0.1.2 Now it works with editable_poly and editable_mesh
	-- 0.1.3 Offset change dialog added
	-- 0.1.4 Now it works with Edit_poly modifier
	-- 0.1.5 Movement is propagated to elements
	-- 0.2 Beta-status and public release
--Last update: 15/12/2012
--Description: Moves the selected sub-object(s) on XY-axis (Screen coordinates) by offset value. Tested in 3ds Max 2012.


global inifile, offset_dialog
global obj, p3, p3_offs

fn logit txt = (
	format "ScreenOffset: %\n" txt
)
fn ScreenOffset = (
	--init
	inifile = ((getdir #userMacros)+"\\ScreenOffset\\screen_offset.ini")
	offs_value = (getinisetting inifile "ScreenOffset" "step" as float)
	offs_tangents = (getinisetting inifile "ScreenOffset" "tangents" as integer)
	dialog_x0 = (getinisetting inifile "ScreenOffset" "dialog_x0" as integer)
	dialog_y0 = (getinisetting inifile "ScreenOffset" "dialog_y0" as integer)

	--action
	if (offset == undefined) then (
		try (closeRolloutFloater offset_dialog) catch ()

		rollout offset_about "About" ( 	
			label offset_title "ScreenOffset v0.2" align:#center
			label offset_created_by "by Alexey Tsepelev" align:#center
			hyperlink offset_url "sumnix.net" address:"http://sumnix.net/" color:(color 0 0 150) hovercolor:(color 0 0 255) visitedcolor:(color 0 0 100) align:#center
			label offset_hint "Moves the selected sub-object(s) on XY-axis (Screen coordinates) by offset value." width:160 height:45 align:#center
		)
		rollout offset_settings "Settings" (
			group "Offset" (
				spinner offset_spinner "Value " tooltip:"Offset value" height:16 align:#center range:[0.01,10,0] type:#float scale:0.01 --controller:""
			)
			group "Spline" (
				checkbox offset_checkbox_tangents "Move tangents" tooltip:"Move tangents with vertices" height:18 align:#center
			)
			on offset_spinner entered do (
				if (setinisetting inifile "ScreenOffset" "step" (offset_spinner.value as string)) then (
					logit "Step is successfully updated"
				) else (
					logit "Cannot write to screen_offset.ini"
					return false
				)
			)
			on offset_checkbox_tangents changed t do (
				if (setinisetting inifile "ScreenOffset" "tangents" ((if t then "1" else "0") as string)) then (
					logit "Tangents are successfully updated"
				) else (
					logit "Cannot write to screen_offset.ini"
					return false
				)
			)
			on offset_settings close do (
				if ((setinisetting inifile "ScreenOffset" "dialog_x0" ((offset_dialog.pos).x as string)) AND (setinisetting inifile "ScreenOffset" "dialog_y0" ((offset_dialog.pos).y as string))) then (
					logit "Data is successfully updated"
				) else (
					logit "Cannot write to screen_offset.ini"
					return false
				)
			)
		)
		offset_dialog = newRolloutFloater "Screen Offset" 200 155
		offset_dialog.pos = [dialog_x0, dialog_y0]
		addrollout offset_settings offset_dialog  
		addrollout offset_about offset_dialog rolledUp:true
		offset_settings.offset_spinner.value = offs_value as float
		offset_settings.offset_checkbox_tangents.checked = (offs_tangents == 1)
		setFocus offset_settings.offset_spinner
	) else (
		if (selection.count == 1 AND offset.count > 1) then (
			obj = selection[1]
			p3_offs = [offset[1]*offs_value, offset[2]*offs_value, offset[3]*offs_value] as point3
			case (superClassOf obj) of (
				Shape: (
					if (classOf obj == SplineShape OR classOf obj == line OR classOf obj == Edit_Spline) do ( -- spline
						in coordsys (inverse (viewport.getTM())) ( -- screen coords
							case subobjectlevel of (
								1: ( --vertex level
									for spline = 1 to (numSplines obj) where (selected = getKnotSelection obj spline) != #() do (
										for knot in selected do (
											p3 = getKnotPoint obj spline knot
											setKnotPoint obj spline knot (p3+=p3_offs)
											type = getKnotType obj spline knot
											if (offs_tangents == 1) then ( --move with tangents
												if (type == #bezier OR type == #bezierCorner) do (
													t1 = getInVec obj spline knot
													t2 = getOutVec obj spline knot
													setInVec obj spline knot (t1+=p3_offs)
													setOutVec obj spline knot (t2+=p3_offs)
												)
											) else ( --move without tangents
												if (type == #bezier) do setKnotType obj spline knot #bezierCorner
											)
										)
									)
								)
								2: ( --segment level
									-- is it even possible using current maxscript's version?
								)
								3: ( --spline level
									-- is it even possible using current maxscript's version?
								)
							)
						)
						updateShape obj
					)
				)
				GeometryClass: ( -- it's a geometry!
					case (classOf obj) of (
						Editable_Poly: ( -- poly
							in coordsys (inverse (viewport.getTM())) ( -- screen coords
								fn move_vertices o v = (
									p3 = polyop.getVert o v
									p3 += p3_offs
									polyop.setVert o v p3
								)
								case subobjectlevel of (
									1: ( --vertex level
										if (selected = getVertSelection obj) != #() do (
											for vertex in selected do move_vertices obj vertex
										)
									)
									2: ( --edge level
										if (selected = getEdgeSelection obj) != #() do (
											for vertex in (polyop.getVertsUsingEdge obj selected) do move_vertices obj vertex
										)
									)
									3: ( --border level
										if (selected = getEdgeSelection obj) != #() do (
											for vertex in (polyop.getVertsUsingEdge obj selected) do move_vertices obj vertex
										)
									)
									4: ( --polygon level
										if (selected = getFaceSelection obj) != #() do (
											for vertex in (polyop.getVertsUsingFace obj selected) do move_vertices obj vertex
										)
									)
									5: ( --element level
										if (selected = getFaceSelection obj) != #() do (
											for vertex in (polyop.getVertsUsingFace obj selected) do move_vertices obj vertex
										)
									)
								)
							)
						)
						Editable_Mesh: ( -- mesh
							in coordsys (inverse (viewport.getTM())) ( -- screen coords
								fn move_vertices o v = (
									if (classOf obj.modifiers[1] == Edit_Mesh) then (
										-- is it even possible using current maxscript's version?
									) else (
										p3 = meshop.getVert o v
										meshop.setVert o v (p3+=p3_offs)
									)
								)
								case subobjectlevel of (
									1: ( --vertex level
										if (selected = getVertSelection obj) != #() do (
											for vertex in selected do move_vertices obj vertex
										)
									)
									2: ( --edge level
										if (selected = getEdgeSelection obj) != #() do (
											for vertex in (meshop.getVertsUsingEdge obj selected) do move_vertices obj vertex
										)
									)
									3: ( --face level
										if (selected = getFaceSelection obj) != #() do (
											for vertex in (meshop.getVertsUsingFace obj selected) do move_vertices obj vertex
										)
									)
									4: ( --polygon level
										if (selected = getFaceSelection obj) != #() do (
											for vertex in (meshop.getVertsUsingFace obj selected) do move_vertices obj vertex
										)
									)
									5: ( --element level
										if (selected = getFaceSelection obj) != #() do (
											for vertex in (meshop.getVertsUsingFace obj selected) do move_vertices obj vertex
										)
									)
								)
							)
						)
						default: ( -- modifiers
							case (classOf obj.modifiers[1]) of (
								Edit_Poly: ( -- edit poly modifier
									ep = obj.modifiers[#Edit_Poly]
									fn move_screen o e i = (
										prev_sys = getRefCoordSys()
										setRefCoordSys #screen
										e.SetOperation #Transform
										e.MoveSelection p3_offs axis:(getTransformAxis o i) -- screen coords
										e.Commit()
										setRefCoordSys prev_sys
									)
									case subobjectlevel of (
										1: ( --vertex level
											if (selected = ep.GetSelection #Vertex) != #() do move_screen obj ep 0
										)
										2: ( --edge level
											if (selected = ep.GetSelection #Edge) != #() do move_screen obj ep 0
										)
										3: ( --border level
											if (selected = ep.GetSelection #Edge) != #() do move_screen obj ep 0
										)
										4: ( --polygon level
											if (selected = ep.GetSelection #Face) != #() do move_screen obj ep 0
										)
										5: ( --element level
											if (selected = ep.GetSelection #Object) != #() do move_screen obj ep 0
										)
									)
								)
							)
						)
					)
					update obj
				)
			)
		) else (
			return false
		)
	)
)

ScreenOffset()
